<?php
class MailQ {
  // (A) PROPERTIES
  public $pdo = null;  // database connection
  public $stmt = null; // sql statement

  // (B) HELPER - CONNECT TO DATABASE
  function connect () : void {
    $this->pdo = new PDO(
      "mysql:host=". DB_HOST .";charset=". DB_CHARSET .";dbname=". DB_NAME,
      DB_USER, DB_PASSWORD, [
      PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
      PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC
    ]);
  }

  // (C) HELPER - DISCONNECT FROM DATABASE
  function disconnect () : void {
    $this->stmt = null;
    $this->pdo = null;
  }

  // (D) HELPER - RUN SQL QUERY
  function query ($sql, $data=null) : void {
    $this->stmt = $this->pdo->prepare($sql);
    $this->stmt->execute($data);
  }

  // (E) ADD EMAIL TO THE QUEUE
  function push ($to, $subject, $message) : void {
    $this->connect();
    $this->query(
      "INSERT INTO `email_queue` (`email_to`, `email_subject`, `email_message`) VALUES (?,?,?)",
      [$to, $subject, $message]
    );
    $this->disconnect();
  }

  // (F) SEND EMAIL TO NEXT IN QUEUE
  // returns true if send ok
  // returns false if send failed
  // returns null if no more in queue
  function pop () {
    // (F1) GET NEXT EMAIL
    $this->connect();
    $this->query("SELECT * FROM `email_queue` ORDER BY `email_queue` ASC LIMIT 1");
    $mail = $this->stmt->fetch();

    // (F2) END OF QUEUE
    if ($mail == null) { $result = null; }

    // (F3) SEND EMAIL
    else {
      echo "Sending to " . $mail["email_to"] . PHP_EOL;
      $result = @mail($mail["email_to"], $mail["email_subject"], $mail["email_message"]);
      $this->query(
        "DELETE FROM `email_queue` WHERE `email_queue`=? AND `email_to`=?",
        [$mail["email_queue"], $mail["email_to"]]
      );
    }

    // (F4) DONE
    $this->disconnect();
    return $result;
  }
}

// (G) SETTINGS - CHANGE TO YOUR OWN!
define("DB_HOST", "localhost");
define("DB_NAME", "test");
define("DB_CHARSET", "utf8mb4");
define("DB_USER", "root");
define("DB_PASSWORD", "");

// (H) MAIL QUEUE ENDPOINT
define("MQ_POINT", "127.0.0.1:8000");

// (I) GO!
$_MQ = new MailQ();